package com.lqkj.camera.task;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.lqkj.business.entity.CameraInfo;
import com.lqkj.business.service.CameraInfoService;
import com.lqkj.camera.model.HikDevice;
import com.lqkj.camera.util.HIKCameraUtils;
import com.lqkj.common.utils.StringUtils;
import com.lqkj.system.service.SysConfigService;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.io.IOException;
import java.net.InetSocketAddress;
import java.net.Socket;
import java.util.ArrayList;
import java.util.List;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * 相机在线状态任务
 */
@Component("cameraOnlineTask")
public class CameraOnlineTask {
    private final static Logger logger = LoggerFactory.getLogger(CameraOnlineTask.class);
    @Autowired
    private CameraInfoService cameraInfoService;
    @Autowired
    private SysConfigService configService;

    public void handleCameraOnline(){
        updateHikOnline(); //更新海康相机状态
        updateRTSPOnline();
    }

    /**
     * RTSP/RTMP相机状态更新
     */
    private void updateRTSPOnline(){
        CameraInfo cameraInfo = new CameraInfo();
        cameraInfo.setCameraType("0");
        List<CameraInfo> cameraInfos = cameraInfoService.selectCameraInfoList(cameraInfo);
        if (cameraInfos.size() > 0){
            for (CameraInfo camera : cameraInfos){
                String ip = CameraOnlineTask.getIp(camera.getUrl());
                String port = CameraOnlineTask.getPort(camera.getUrl());
                if (StringUtils.isNotEmpty(ip) && StringUtils.isNotEmpty(port)){
                    Socket rtspSocket = new Socket();
                    try {
                        camera.setOnlineStatus(true);
                        rtspSocket.connect(new InetSocketAddress(ip, Integer.parseInt(port.trim())), 2000);
                    } catch (IOException e) {
                        camera.setOnlineStatus(false);
                        logger.error(camera.getName() + "-" + camera.getUrl() + "-离线");
                    }finally {
                        cameraInfoService.updateOnlineStatusById(camera);
                        try {
                            rtspSocket.close();
                        } catch (IOException e) {

                        }
                    }
                }
            }


        }

    }

    /**
     * 海康相机状态更新
     */
    private void updateHikOnline(){
        CameraInfo cameraInfo = new CameraInfo();
        cameraInfo.setCameraType("1");
        List<CameraInfo> cameraInfos = cameraInfoService.selectCameraInfoList(cameraInfo);
        if (cameraInfos.size() > 0){
            List<HikDevice> hikDevices = new ArrayList<>();
            String hikHost = configService.selectConfigByKey("hik.camera.host");
            String appKey = configService.selectConfigByKey("hik.camera.appKey");
            String appSecret = configService.selectConfigByKey("hik.camera.appSecret");
            String httpProtocol = configService.selectConfigByKey("hik.camera.httpProtocol");
            if (StringUtils.isNotEmpty(hikHost)
                && StringUtils.isNotEmpty(appKey)
                && StringUtils.isNotEmpty(appSecret)
                && StringUtils.isNotEmpty(httpProtocol)){
                int page  = 1, pageSize = 1000;
                boolean hasNext = true;
                while (hasNext){
                    String res = null;
                    try {
                        res = HIKCameraUtils.getDefaultInstance(hikHost, appKey, appSecret, httpProtocol)
                                .pageQueryCamerasWithStatus(pageSize, page);
                    } catch (Exception e) {
                        logger.error("海康平台异常");
                    }
                    if (StringUtils.isEmpty(res)){
                        hasNext = false;
                    }else{
                        JSONObject jsonObject = JSON.parseObject(res);
                        JSONObject data = jsonObject.getJSONObject("data");
                        int totalPage = data.getInteger("totalPage");
                        if (page < totalPage){
                            page += 1;
                        }else{
                            hasNext = false;
                        }
                        hikDevices.addAll(JSON.parseArray(data.getString("list"), HikDevice.class));
                    }
                }
            }else{
                logger.warn("海康平台对接参数未配置");
            }
            for (HikDevice hikDevice : hikDevices){
                CameraInfo info = new CameraInfo();
                info.setCameraIndexCode(hikDevice.getIndexCode());
                info.setOnlineStatus(hikDevice.getOnline() == 0 ? false : true);
                cameraInfoService.updateOnlineStatusByIndexCode(info);
            }
        }
    }

    private static String getIp(String url){
        String ip = "";
        Pattern pattern = Pattern.compile("(\\d+\\.\\d+\\.\\d+\\.\\d+)");
        Matcher matcher = pattern.matcher(url);
        while(matcher.find()) {
            ip = matcher.group(1);
        }
        return ip;
    }
    private static String getPort(String url){
        String port = "";
        Pattern pattern = Pattern.compile("(\\d+\\.\\d+\\.\\d+\\.\\d+)\\:(\\d+)");
        Matcher matcher = pattern.matcher(url);
        while(matcher.find()) {
            port = matcher.group(2);
        }
        return port;
    }

}
